@@ -11,10 +11,10 @@ lm.gls <-
     m$W <- m$inverse <- m$method <- m$model <- m$x <-
         m$y <- m$contrasts <- m$... <- NULL
     m[[1]] <- as.name("model.frame")
-    m <- eval(m, parent.frame())
+    m <- eval.parent(m)
     if(method == "model.frame") return(m)
     Terms <- attr(m, "terms")
-    Y <- model.extract(m, response)
+    Y <- model.response(m)
     X <- model.matrix(Terms, m, contrasts)
     xvars <- as.character(attr(Terms, "variables"))[-1]
     if ((yvar <- attr(Terms, "response")) > 0) xvars <- xvars[-yvar]
