@@ -1,12 +1,14 @@
 # file MASS/sammon.q
-# copyright (C) 1994-2000 W. N. Venables and B. D. Ripley
+# copyright (C) 1994-2003 W. N. Venables and B. D. Ripley
 #
 sammon <- function(d, y= cmdscale(d, k), k=2, niter=100, trace=TRUE,
                    magic=0.2, tol=1e-4)
 {
    call <- match.call()
-   if(any(!is.finite(as.vector(d))))
-      stop("NAs/Infs not allowed in d")
+   if(any(is.infinite(as.vector(d))))
+      stop("Infs not allowed in d")
+   if(any(is.na(d)) && missing(y))
+       stop("An initial configuration must be supplied with NAs in d")
    if(is.null(n <- attr(d, "Size"))) {
       x <- as.matrix(d)
       if((n <- nrow(x)) != ncol(x))
@@ -17,14 +19,20 @@ sammon <- function(d, y= cmdscale(d, k), k=2, niter=100, trace=TRUE,
       x[row(x) > col(x)] <- d
       x <- x + t(x)
    }
-   if (any(ab <- x[row(x) < col(x)]<=0)) {
-	aa <- cbind(as.vector(row(x)), as.vector(col(x)))[row(x) < col(x),]
-	aa <- aa[ab,,drop=FALSE]
-	stop(paste("zero or negative distance between objects", aa[1,1],
-	 "and", aa[1,2]))
-	}
+   ab <- x[row(x) < col(x)]<=0
+   if (any(ab %in% TRUE)) {
+       aa <- cbind(as.vector(row(x)), as.vector(col(x)))[row(x) < col(x),]
+       aa <- aa[ab,,drop=FALSE]
+       stop(paste("zero or negative distance between objects", aa[1,1],
+                  "and", aa[1,2]))
+   }
+   nas <- is.na(x)
+   diag(nas) <- FALSE  # diag never used
+   if(any(rowSums(!nas) < 2)) stop("not enough non-missing data")
+
    if(!is.matrix(y)) stop("y must be a matrix")
    if(any(dim(y) != c(n, k)) ) stop("invalid initial configuration")
+   if(any(!is.finite(y))) stop("initial configuration must be complete")
    storage.mode(x) <- "double"
    storage.mode(y) <- "double"
    if(!is.loaded(symbol.C("VR_sammon")))
@@ -38,12 +46,11 @@ sammon <- function(d, y= cmdscale(d, k), k=2, niter=100, trace=TRUE,
       e = double(1),
       as.integer(trace),
       as.double(magic),
-      as.double(tol)
-      , PACKAGE = "MASS"
+      as.double(tol),
+      NAOK = TRUE, PACKAGE = "MASS"
       )
    points <- z$y
    rn <- if(is.matrix(d)) rownames(d) else names(d)
    dimnames(points) <- list(rn, NULL)
    list(points=points, stress=z$e, call=call)
 }
-
